@@ -110,6 +110,12 @@ class ATagDTO:
     image: Optional[AImageDTO] = None
 
 
+@dataclass_json
+@dataclass
+class TagDTO(ATagDTO):
+    imageUuid: Optional[str] = None
+
+
 @dataclass_json
 @dataclass
 class ABoundingBoxDTO:
@@ -165,7 +171,7 @@ class Authentication:
 
 @dataclass_json
 @dataclass
-class BBoundingBoxDTO(ABoundingBoxDTO):
+class BoundingBoxDTO(ABoundingBoxDTO):
     imageUuid: Optional[str] = None
 
 
@@ -209,7 +215,7 @@ class ByContributorCount:
 
 @dataclass_json
 @dataclass
-class CDarwinCore:
+class BDarwinCore:
     uuid: Optional[str] = None
     recordType: Optional[str] = None
     basisOfRecord: Optional[str] = None
@@ -234,7 +240,7 @@ class CDarwinCore:
 
 @dataclass_json
 @dataclass
-class CImageSetUploadDTO:
+class BImageSetUploadDTO:
     uuid: Optional[str] = None
     localPath: Optional[str] = None
     remoteUri: Optional[str] = None
@@ -246,7 +252,7 @@ class CImageSetUploadDTO:
     statusUpdateTimestamp: Optional[str] = None
     rejectionReason: Optional[str] = None
     rejectionDetails: Optional[str] = None
-    darwinCore: Optional[CDarwinCore] = None
+    darwinCore: Optional[BDarwinCore] = None
     createdTimestamp: Optional[str] = None
     lastUpdatedTimestamp: Optional[str] = None
 
